/*
 * Copyright 2004-2006 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package javax.faces.internal;

import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.NoSuchElementException;

/**
 * @author shot
 * 
 * This class might be changed without notice. Please do not use it
 * excluding the JSF specification part.
 */
public class ComponentFacetAndChildrenIterator implements Iterator {

    private Iterator facetItr_ = null;

    private Iterator childrenItr_ = null;

    public ComponentFacetAndChildrenIterator(Map facetMap, List childrenList) {
        if (facetMap != null) {
            facetItr_ = facetMap.values().iterator();
        }
        if (childrenList != null) {
            childrenItr_ = childrenList.iterator();
        }
    }

    public void remove() {
        throw new UnsupportedOperationException("remove");
    }

    public boolean hasNext() {
        return (facetHasNext() || childrenHasNext());
    }

    public Object next() {
        if (facetHasNext()) {
            return facetItr_.next();
        }
        if (childrenHasNext()) {
            return childrenItr_.next();
        }
        throw new NoSuchElementException();
    }

    private boolean facetHasNext() {
        return (facetItr_ != null && facetItr_.hasNext());
    }

    private boolean childrenHasNext() {
        return (childrenItr_ != null && childrenItr_.hasNext());
    }
}
